@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -67,7 +69,14 @@ let _hctl = Hashtbl.create 101
 (* --------------------------------------------------------------------- *)
 let sp_of_file2 file iso   =
   Common.memoized _hparse (file, iso) (fun () ->
-    Parse_cocci.process file iso false)
+    let (_,xs,_,_,_,_,_,_) as res = Parse_cocci.process file iso false in
+    (match Prepare_ocamlcocci.prepare file xs with
+      None -> ()
+    | Some ocaml_script_file ->
+        (* compile file *)
+	Prepare_ocamlcocci.load_file ocaml_script_file;
+	Prepare_ocamlcocci.clean_file ocaml_script_file);
+    res)
 let sp_of_file file iso    =
   Common.profile_code "parse cocci" (fun () -> sp_of_file2 file iso)
 
@@ -165,13 +174,15 @@ let fix_sgrep_diffs l =
 	then
 	  (match Str.split (Str.regexp " ") s with
 	    bef::min::pl::aft ->
-	      (match Str.split (Str.regexp ",") pl with
-		[n1;n2] ->
-		  let n2 = int_of_string n2 in
-		  (Printf.sprintf "%s %s %s,%d %s" bef min n1 (n2-n)
-		     (String.concat " " aft))
-		  :: loop1 0 ss
-	      | _ -> failwith "bad + line information")
+	      let (n1,n2) =
+		match Str.split (Str.regexp ",") pl with
+		  [n1;n2] -> (n1,n2)
+		| [n1] -> (n1,"1")
+		| _ -> failwith "bad + line information" in
+	      let n2 = int_of_string n2 in
+	      (Printf.sprintf "%s %s %s,%d %s" bef min n1 (n2-n)
+		 (String.concat " " aft))
+	      :: loop1 0 ss
 	  | _ -> failwith "bad @@ information")
 	else s :: loop1 n ss in
   let rec loop2 n = function
@@ -183,18 +194,21 @@ let fix_sgrep_diffs l =
 	then
 	  (match Str.split (Str.regexp " ") s with
 	    bef::min::pl::aft ->
-	      (match (Str.split (Str.regexp ",") min,
-		      Str.split (Str.regexp ",") pl) with
-		([_;m2],[n1;n2]) ->
-		  let n1 =
-		    int_of_string
-		      (String.sub n1 1 ((String.length n1)-1)) in
-		  let m2 = int_of_string m2 in
-		  let n2 = int_of_string n2 in
-		  (Printf.sprintf "%s %s +%d,%d %s" bef min (n1-n) n2
-		     (String.concat " " aft))
-		  :: loop2 (n+(m2-n2)) ss
-	      | _ -> failwith "bad -/+ line information")
+	      let (m2,n1,n2) =
+		match (Str.split (Str.regexp ",") min,
+		       Str.split (Str.regexp ",") pl) with
+		  ([_;m2],[n1;n2]) -> (m2,n1,n2)
+		| ([_],[n1;n2]) -> ("1",n1,n2)
+		| ([_;m2],[n1]) -> (m2,n1,"1")
+		| ([_],[n1]) -> ("1",n1,"1")
+		| _ -> failwith "bad -/+ line information" in
+	      let n1 =
+		int_of_string (String.sub n1 1 ((String.length n1)-1)) in
+	      let m2 = int_of_string m2 in
+	      let n2 = int_of_string n2 in
+	      (Printf.sprintf "%s %s +%d,%d %s" bef min (n1-n) n2
+		 (String.concat " " aft))
+	      :: loop2 (n+(m2-n2)) ss
 	  | _ -> failwith "bad @@ information")
 	else s :: loop2 n ss in
   loop2 0 (List.rev (loop1 0 l))
@@ -436,46 +450,44 @@ let worth_trying cfiles tokens =
   (* drop the following line for a list of list by rules.  since we don't
      allow multiple minirules, all the tokens within a rule should be in
      a single CFG entity *)
-  let tokens = Common.union_all tokens in
-  if not !Flag_cocci.windows && not (null tokens)
-  then
+  match (!Flag_cocci.windows,tokens) with
+    (true,_) | (_,None) -> true
+  | (_,Some tokens) ->
    (* could also modify the code in get_constants.ml *)
-    let tokens = tokens +> List.map (fun s ->
-      match () with
-      | _ when s =~ "^[A-Za-z_][A-Za-z_0-9]*$" ->
-          "\\b" ^ s ^ "\\b"
+      let tokens = tokens +> List.map (fun s ->
+	match () with
+	| _ when s =~ "^[A-Za-z_][A-Za-z_0-9]*$" ->
+            "\\b" ^ s ^ "\\b"
 
-      | _ when s =~ "^[A-Za-z_]" ->
-          "\\b" ^ s
+	| _ when s =~ "^[A-Za-z_]" ->
+            "\\b" ^ s
 
-      | _ when s =~ ".*[A-Za-z_]$" ->
-          s ^ "\\b"
-      | _ -> s
+	| _ when s =~ ".*[A-Za-z_]$" ->
+            s ^ "\\b"
+	| _ -> s
 
-    ) in
-    let com = sprintf "egrep -q '(%s)' %s" (join "|" tokens) (join " " cfiles)
-    in
-    (match Sys.command com with
-    | 0 (* success *) -> true
-    | _ (* failure *) ->
-	(if !Flag.show_misc
-	then Printf.printf "grep failed: %s\n" com);
-	false (* no match, so not worth trying *)
-    )
-  else true
+      ) in
+      let com = sprintf "egrep -q '(%s)' %s" (join "|" tokens) (join " " cfiles)
+      in
+      (match Sys.command com with
+      | 0 (* success *) -> true
+      | _ (* failure *) ->
+	  (if !Flag.show_misc
+	  then Printf.printf "grep failed: %s\n" com);
+	  false (* no match, so not worth trying *))
 
-let check_macro_in_sp_and_adjust tokens =
-  let tokens = Common.union_all tokens in
-  tokens +> List.iter (fun s ->
-    if Hashtbl.mem !Parse_c._defs s
-    then begin
-      if !Flag_cocci.verbose_cocci then begin
-        pr2 "warning: macro in semantic patch was in macro definitions";
-        pr2 ("disabling macro expansion for " ^ s);
-      end;
-      Hashtbl.remove !Parse_c._defs s
-    end
-  )
+let check_macro_in_sp_and_adjust = function
+    None -> ()
+  | Some tokens ->
+      tokens +> List.iter (fun s ->
+	if Hashtbl.mem !Parse_c._defs s
+	then begin
+	  if !Flag_cocci.verbose_cocci then begin
+            pr2 "warning: macro in semantic patch was in macro definitions";
+            pr2 ("disabling macro expansion for " ^ s);
+	  end;
+	  Hashtbl.remove !Parse_c._defs s
+	end)
 
 
 let contain_loop gopt =
@@ -574,15 +586,20 @@ let (includes_to_parse:
             | Ast_c.Local xs ->
 		let relpath = Common.join "/" xs in
 		let f = Filename.concat dir (relpath) in
+		if (Sys.file_exists f) then
+		  Some f
+		else
+		  if !Flag_cocci.relax_include_path
 	      (* for our tests, all the files are flat in the current dir *)
-		if not (Sys.file_exists f) && !Flag_cocci.relax_include_path
-		then
-		  let attempt2 = Filename.concat dir (Common.last xs) in
-		  if not (Sys.file_exists f) && all_includes
 		  then
-		    interpret_include_path relpath
-		  else Some attempt2
-		else Some f
+		    let attempt2 = Filename.concat dir (Common.last xs) in
+		      if not (Sys.file_exists attempt2) && all_includes
+		      then
+			interpret_include_path relpath
+		      else Some attempt2
+		  else
+		    if all_includes then interpret_include_path relpath
+		    else None
 
             | Ast_c.NonLocal xs ->
 		let relpath = Common.join "/" xs in
@@ -748,7 +765,12 @@ type toplevel_c_info = {
 }
 
 type toplevel_cocci_info_script_rule = {
-  scr_ast_rule: string * (string * Ast_cocci.meta_name) list * string;
+  scr_rulename: string;
+  scr_ast_rule:
+      string *
+      (Ast_cocci.script_meta_name * Ast_cocci.meta_name *
+	 Ast_cocci.metavar) list *
+      string;
   language: string;
   scr_dependencies: Ast_cocci.dependency;
   scr_ruleid: int;
@@ -784,7 +806,7 @@ type toplevel_cocci_info =
   | FinalScriptRuleCocciInfo of toplevel_cocci_info_script_rule
   | CocciRuleCocciInfo of toplevel_cocci_info_cocci_rule
 
-type cocci_info = toplevel_cocci_info list * string list list (* tokens *)
+type cocci_info = toplevel_cocci_info list * string list option (* tokens *)
 
 type kind_file = Header | Source
 type file_info = {
@@ -843,9 +865,10 @@ let python_code =
     local_python_code ^
     "cocci = Cocci()\n"
 
-let make_init rulenb lang deps code =
+let make_init name rulenb lang deps code =
   let mv = [] in
   {
+  scr_rulename = name;
   scr_ast_rule = (lang, mv, code);
   language = lang;
   scr_dependencies = deps;
@@ -876,9 +899,10 @@ let prepare_cocci ctls free_var_lists negated_pos_lists
       then failwith "not handling multiple minirules";
 
       match ast with
-        Ast_cocci.ScriptRule (lang,deps,mv,code) ->
+        Ast_cocci.ScriptRule (name,lang,deps,mv,code) ->
           let r =
           {
+	    scr_rulename = name;
             scr_ast_rule = (lang, mv, code);
             language = lang;
             scr_dependencies = deps;
@@ -886,13 +910,14 @@ let prepare_cocci ctls free_var_lists negated_pos_lists
             script_code = code;
           }
           in ScriptRuleCocciInfo r
-      | Ast_cocci.InitialScriptRule (lang,deps,code) ->
-	  let r = make_init rulenb lang deps code in
+      | Ast_cocci.InitialScriptRule (name,lang,deps,code) ->
+	  let r = make_init name rulenb lang deps code in
 	    InitialScriptRuleCocciInfo r
-      | Ast_cocci.FinalScriptRule (lang,deps,code) ->
+      | Ast_cocci.FinalScriptRule (name,lang,deps,code) ->
 	  let mv = [] in
           let r =
           {
+	    scr_rulename = name;
             scr_ast_rule = (lang, mv, code);
             language = lang;
             scr_dependencies = deps;
@@ -1140,9 +1165,40 @@ let merge_env new_e old_e =
       ([],old_e) new_e in
   old_e @ (List.rev ext)
 
-let apply_python_rule r cache newes e rules_that_have_matched
-    rules_that_have_ever_matched =
-  Common.profile_code "python" (fun () ->
+let contains_binding e (_,(r,m),_) =
+  try
+    let _ = List.find (function ((re, rm), _) -> r =*= re && m =$= rm) e in
+    true
+  with Not_found -> false
+
+let python_application mv ve r =
+  let mv =
+    List.map
+      (function
+	  ((Some x,None),y,z) -> (x,y,z)
+	| _ ->
+	    failwith
+	      (Printf.sprintf "unexpected ast metavar in rule %s"
+		 r.scr_rulename))
+      mv in
+  try
+    Pycocci.build_classes (List.map (function (x,y) -> x) ve);
+    Pycocci.construct_variables mv ve;
+    let _ = Pycocci.pyrun_simplestring (local_python_code ^r.script_code) in
+    !Pycocci.inc_match
+  with Pycocci.Pycocciexception ->
+    (pr2 ("Failure in " ^ r.scr_rulename);
+     raise Pycocci.Pycocciexception)
+
+let ocaml_application mv ve r =
+  try
+    Run_ocamlcocci.run mv ve r.scr_rulename r.script_code;
+    !Coccilib.inc_match
+  with e -> (pr2 ("Failure in " ^ r.scr_rulename); raise e)
+
+let apply_script_rule r cache newes e rules_that_have_matched
+    rules_that_have_ever_matched script_application =
+  Common.profile_code r.language (fun () ->
   show_or_not_scr_rule_name r.scr_ruleid;
   if not(interpret_dependencies rules_that_have_matched
 	   !rules_that_have_ever_matched r.scr_dependencies)
@@ -1158,49 +1214,45 @@ let apply_python_rule r cache newes e rules_that_have_matched
     begin
       let (_, mv, _) = r.scr_ast_rule in
       let ve =
-	(List.map (function (n,v) -> (("virtual",n),Ast_c.MetaIdVal v))
+	(List.map (function (n,v) -> (("virtual",n),Ast_c.MetaIdVal (v,[])))
 	   !Flag.defined_virtual_env) @ e in
-      let not_bound x = not (Pycocci.contains_binding ve x) in
+      let not_bound x = not (contains_binding ve x) in
       (match List.filter not_bound mv with
 	[] ->
 	  let relevant_bindings =
 	    List.filter
 	      (function ((re,rm),_) ->
-		List.exists (function (_,(r,m)) -> r =*= re && m =$= rm) mv)
+		List.exists (function (_,(r,m),_) -> r =*= re && m =$= rm) mv)
 	      e in
-	  let new_cache =
-	    if List.mem relevant_bindings cache
-	    then
-	      begin
-		print_dependencies
-		  "dependencies for script satisfied, but cached:"
-		  rules_that_have_matched
-		  !rules_that_have_ever_matched
-		  r.scr_dependencies;
-		show_or_not_binding "in" e;
-		cache
-	      end
-	    else
-	      begin
-		print_dependencies "dependencies for script satisfied:"
-		  rules_that_have_matched
-		  !rules_that_have_ever_matched
-		  r.scr_dependencies;
-		show_or_not_binding "in" e;
-		Pycocci.build_classes (List.map (function (x,y) -> x) ve);
-		Pycocci.construct_variables mv ve;
-		let _ =
-		  Pycocci.pyrun_simplestring
-		    (local_python_code ^r.script_code) in
-		relevant_bindings :: cache
-	      end in
-	  if !Pycocci.inc_match
-	  then (new_cache, merge_env [(e, rules_that_have_matched)] newes)
-	  else (new_cache, newes)
+	  if List.mem relevant_bindings cache
+	  then
+	    begin
+	      print_dependencies
+		"dependencies for script satisfied, but cached:"
+		rules_that_have_matched
+		!rules_that_have_ever_matched
+		r.scr_dependencies;
+	      show_or_not_binding "in" e;
+	      (* env might be bigger than what was cached against, so have to
+		 merge with newes anyway *)
+	      (cache,merge_env [(e, rules_that_have_matched)] newes)
+	    end
+	  else
+	    begin
+	      print_dependencies "dependencies for script satisfied:"
+		rules_that_have_matched
+		!rules_that_have_ever_matched
+		r.scr_dependencies;
+	      show_or_not_binding "in" e;
+	      let new_cache = relevant_bindings :: cache in
+	      if script_application mv ve r
+	      then (new_cache, merge_env [(e, rules_that_have_matched)] newes)
+	      else (new_cache, newes)
+	    end
       |	unbound ->
 	  (if !Flag_cocci.show_dependencies
 	  then
-	    let m2c (_,(r,x)) = r^"."^x in
+	    let m2c (_,(r,x),_) = r^"."^x in
 	    pr2 (Printf.sprintf "script not applied: %s not bound"
 		   (String.concat ", " (List.map m2c unbound))));
 	  (cache, merge_env [(e, rules_that_have_matched)] newes))
@@ -1511,7 +1563,7 @@ let rec bigloop2 rs (ccs: file_info list) =
             let (l,mv,code) = r.scr_ast_rule in
 	    let deps = r.scr_dependencies in
             Pretty_print_cocci.unparse
-	      (Ast_cocci.ScriptRule (l,deps,mv,code)));
+	      (Ast_cocci.ScriptRule ("",l,deps,mv,code)));
 	end;
 
 	if !Flag.show_misc then print_endline "RESULT =";
@@ -1522,8 +1574,11 @@ let rec bigloop2 rs (ccs: file_info list) =
               function (e, rules_that_have_matched) ->
 		match r.language with
                   "python" ->
-		    apply_python_rule r cache newes e rules_that_have_matched
-		      rules_that_have_ever_matched
+		    apply_script_rule r cache newes e rules_that_have_matched
+		      rules_that_have_ever_matched python_application
+                | "ocaml" ->
+		    apply_script_rule r cache newes e rules_that_have_matched
+		      rules_that_have_ever_matched ocaml_application
 		| "test" ->
 		    concat_headers_and_c !ccs +> List.iter (fun (c,_) ->
 		      if c.flow <> None
@@ -1532,8 +1587,7 @@ let rec bigloop2 rs (ccs: file_info list) =
 		    (cache, newes)
 		| _ ->
                     Printf.printf "Unknown language: %s\n" r.language;
-                    (cache, newes)
-		      )
+                    (cache, newes))
             ([],[]) !es in
 
         es := (if newes = [] then init_es else newes);
@@ -1559,11 +1613,14 @@ let rec bigloop2 rs (ccs: file_info list) =
 let bigloop a b =
   Common.profile_code "bigloop" (fun () -> bigloop2 a b)
 
+type init_final = Initial | Final
+
 let initial_final_bigloop2 ty rebuild r =
   if !Flag_cocci.show_ctl_text then
     begin
       Common.pr_xxxxxxxxxxxxxxxxx ();
-      pr (ty ^ ": " ^ r.language);
+      pr ((match ty with Initial -> "initial" | Final -> "final") ^ ": " ^
+	  r.language);
       Common.pr_xxxxxxxxxxxxxxxxx ();
 
       adjust_pp_with_indent (fun () ->
@@ -1575,7 +1632,13 @@ let initial_final_bigloop2 ty rebuild r =
     "python" ->
       (* include_match makes no sense in an initial or final rule, although
 	 we have no way to prevent it *)
-      let _ = apply_python_rule r [] [] [] [] (ref []) in
+      let _ = apply_script_rule r [] [] [] [] (ref []) python_application in
+      ()
+  | "ocaml" when ty = Initial -> () (* nothing to do *)
+  | "ocaml" ->
+      (* include_match makes no sense in an initial or final rule, although
+	 we have no way to prevent it *)
+      let _ = apply_script_rule r [] [] [] [] (ref []) ocaml_application in
       ()
   | _ ->
       Printf.printf "Unknown language for initial/final script: %s\n"
@@ -1602,7 +1665,8 @@ let pre_engine2 (coccifile, isofile) =
     else Some isofile in
 
   (* useful opti when use -dir *)
-  let (metavars,astcocci,free_var_lists,negated_pos_lists,used_after_lists,
+  let (metavars,astcocci,
+       free_var_lists,negated_pos_lists,used_after_lists,
        positions_lists,toks,_) =
       sp_of_file coccifile isofile in
   let ctls = ctls_of_ast astcocci used_after_lists positions_lists in
@@ -1635,13 +1699,15 @@ let pre_engine2 (coccifile, isofile) =
 	 function
 	     InitialScriptRuleCocciInfo(r) ->
 	       (if List.mem r.language languages
-		then failwith ("double initializer found for "^r.language));
+		then
+		 failwith
+		   ("double initializer found for "^r.language));
 	       if interpret_dependencies [] [] r.scr_dependencies
 	       then
 		 begin
-		   initial_final_bigloop "initial"
+		   initial_final_bigloop Initial
 		     (fun (x,_,y) -> fun deps ->
-		       Ast_cocci.InitialScriptRule(x,deps,y))
+		       Ast_cocci.InitialScriptRule(r.scr_rulename,x,deps,y))
 		     r;
 		   r.language::languages
 		 end
@@ -1655,10 +1721,10 @@ let pre_engine2 (coccifile, isofile) =
       used_languages
   in
     List.iter (fun lgg ->
-		 initial_final_bigloop "initial"
+		 initial_final_bigloop Initial
 		   (fun (x,_,y) -> fun deps ->
-		     Ast_cocci.InitialScriptRule(x,deps,y))
-		   (make_init (-1) lgg Ast_cocci.NoDep "");
+		     Ast_cocci.InitialScriptRule("",x,deps,y))
+		   (make_init "" (-1) lgg Ast_cocci.NoDep "");
 	      )
       uninitialized_languages;
 
@@ -1675,8 +1741,11 @@ let full_engine2 (cocci_infos,toks) cfiles =
   if !Flag_cocci.worth_trying_opt && not (worth_trying cfiles toks)
   then
     begin
-      pr2 ("No matches found for " ^ (Common.join " " (Common.union_all toks))
-	   ^ "\nSkipping:" ^ (Common.join " " cfiles));
+      (match toks with
+	None -> ()
+      | Some toks ->
+	  pr2 ("No matches found for " ^ (Common.join " " toks)
+	       ^ "\nSkipping:" ^ (Common.join " " cfiles)));
       cfiles +> List.map (fun s -> s, None)
     end
   else
@@ -1736,8 +1805,9 @@ let post_engine2 (cocci_infos,_) =
 	    FinalScriptRuleCocciInfo(r) ->
 	      (if List.mem r.language languages
 	      then failwith ("double finalizer found for "^r.language));
-	      initial_final_bigloop "final"
-		(fun (x,_,y) -> fun deps -> Ast_cocci.FinalScriptRule(x,deps,y))
+	      initial_final_bigloop Final
+		(fun (x,_,y) -> fun deps ->
+		  Ast_cocci.FinalScriptRule(r.scr_rulename,x,deps,y))
 		r;
 	      r.language::languages
 	  | _ -> languages)
