@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -218,7 +220,8 @@ let elim_opt =
 
     | (Ast.Dots(_,_,_,_)::Ast.OptStm(stm)::(Ast.Dots(_,_,_,_) as u)::urest,
        d0::s::d1::rest)
-    | (Ast.Nest(_,_,_,_,_)::Ast.OptStm(stm)::(Ast.Dots(_,_,_,_) as u)::urest,
+    | (Ast.Nest(_,_,_,_,_,_,_)::Ast.OptStm(stm)::(Ast.Dots(_,_,_,_) as u)
+       ::urest,
        d0::s::d1::rest) -> (* why no case for nest as u? *)
 	 let l = Ast.get_line stm in
 	 let new_rest1 = stm :: (dots_list (u::urest) (d1::rest)) in
@@ -323,7 +326,7 @@ let elim_opt =
 	     Ast.inherited = inh_both;
 	     Ast.saved_witness = saved_both}]
 
-    | ([Ast.Nest(_,_,_,_,_);Ast.OptStm(stm)],[d1;_]) ->
+    | ([Ast.Nest(_,_,_,_,_,_,_);Ast.OptStm(stm)],[d1;_]) ->
 	let l = Ast.get_line stm in
 	let rw = Ast.rewrap stm in
 	let rwd = Ast.rewrap stm in
@@ -444,11 +447,16 @@ let contains_modif =
 	bind (mcode r ((),(),bef,Ast.NoMetaPos)) res
     | Ast.Decl(bef,_,decl) -> bind (mcode r ((),(),bef,Ast.NoMetaPos)) res
     | _ -> res in
+  let init r k i =
+    let res = k i in
+    match Ast.unwrap i with
+      Ast.InitList(allminus,_,_,_,_) -> allminus or res
+    | _ -> res in
   let recursor =
     V.combiner bind option_default
       mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
       do_nothing do_nothing do_nothing do_nothing
-      do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
+      do_nothing do_nothing do_nothing do_nothing init do_nothing
       do_nothing rule_elem do_nothing do_nothing do_nothing do_nothing in
   recursor.V.combiner_rule_elem
 
@@ -605,7 +613,7 @@ and get_before_e s a =
   match Ast.unwrap s with
     Ast.Dots(d,w,_,aft) ->
       (Ast.rewrap s (Ast.Dots(d,get_before_whencode w,a,aft)),a)
-  | Ast.Nest(stmt_dots,w,multi,_,aft) ->
+  | Ast.Nest(starter,stmt_dots,ender,w,multi,_,aft) ->
       let w = get_before_whencode w in
       let (sd,_) = get_before stmt_dots a in
       (*let a =
@@ -626,7 +634,8 @@ and get_before_e s a =
 		| _ -> true)
 	    | _ -> true)
 	  a in*)
-      (Ast.rewrap s (Ast.Nest(sd,w,multi,a,aft)),[Ast.Other_dots stmt_dots])
+      (Ast.rewrap s (Ast.Nest(starter,sd,ender,w,multi,a,aft)),
+       [Ast.Other_dots stmt_dots])
   | Ast.Disj(stmt_dots_list) ->
       let (dsl,dsla) =
 	List.split (List.map (function e -> get_before e a) stmt_dots_list) in
@@ -711,7 +720,7 @@ and get_after_e s a =
   match Ast.unwrap s with
     Ast.Dots(d,w,bef,_) ->
       (Ast.rewrap s (Ast.Dots(d,get_after_whencode a w,bef,a)),a)
-  | Ast.Nest(stmt_dots,w,multi,bef,_) ->
+  | Ast.Nest(starter,stmt_dots,ender,w,multi,bef,_) ->
       let w = get_after_whencode a w in
       let (sd,_) = get_after stmt_dots a in
       (*let a =
@@ -732,7 +741,8 @@ and get_after_e s a =
 		| _ -> true)
 	    | _ -> true)
 	  a in*)
-      (Ast.rewrap s (Ast.Nest(sd,w,multi,bef,a)),[Ast.Other_dots stmt_dots])
+      (Ast.rewrap s (Ast.Nest(starter,sd,ender,w,multi,bef,a)),
+       [Ast.Other_dots stmt_dots])
   | Ast.Disj(stmt_dots_list) ->
       let (dsl,dsla) =
 	List.split (List.map (function e -> get_after e a) stmt_dots_list) in
@@ -747,7 +757,7 @@ and get_after_e s a =
 	    (function
 		Ast.Other x ->
 		  (match Ast.unwrap x with
-		    Ast.Dots(_,_,_,_) | Ast.Nest(_,_,_,_,_) ->
+		    Ast.Dots(_,_,_,_) | Ast.Nest(_,_,_,_,_,_,_) ->
 		      failwith
 			"dots/nest not allowed before and after stmt metavar"
 		  | _ -> ())
@@ -755,7 +765,7 @@ and get_after_e s a =
 		  (match Ast.undots x with
 		    x::_ ->
 		      (match Ast.unwrap x with
-			Ast.Dots(_,_,_,_) | Ast.Nest(_,_,_,_,_) ->
+			Ast.Dots(_,_,_,_) | Ast.Nest(_,_,_,_,_,_,_) ->
 			  failwith
 			    ("dots/nest not allowed before and after stmt "^
 			     "metavar")
@@ -1530,9 +1540,9 @@ let rec dots_and_nests plus nest whencodes bef aft dotcode after label
   plus_modifier
     (dots_au is_strict ((after = Tail) or (after = VeryEnd))
        label (guard_to_strict guard) wrapcode just_nest
-      (ctl_and_ns dotcode
-	 (ctl_and_ns (ctl_and_ns (ctl_not bef_aft) ornest) labelled))
-      aft ender quantifier)
+       (ctl_and_ns dotcode
+	   (ctl_and_ns (ctl_and_ns (ctl_not bef_aft) ornest) labelled))
+       aft ender quantifier)
 
 and get_whencond_exps e =
   match Ast.unwrap e with
@@ -1585,9 +1595,12 @@ and whencond_false e label guard quantified =
   let e1 = get_whencond_exps e in
   let (if_headers, while_headers, for_headers) =
     make_whencond_headers e e1 label guard quantified in
+  (* if with else *)
   ctl_or (ctl_and CTL.NONSTRICT (falsepred label) (ctl_back_ex if_headers))
-    (ctl_and CTL.NONSTRICT (loopfallpred label)
-       (ctl_or (ctl_back_ex if_headers)
+    (* if without else *)
+    (ctl_or (ctl_and CTL.NONSTRICT (fallpred label) (ctl_back_ex if_headers))
+       (* failure of loop test *)
+       (ctl_and CTL.NONSTRICT (loopfallpred label)
 	  (ctl_or (ctl_back_ex while_headers) (ctl_back_ex for_headers))))
 
 (* --------------------------------------------------------------------- *)
@@ -1893,7 +1906,7 @@ and statement stmt after quantified minus_quantified
 		  llabel slabel true guard)
 	      stmt_dots_list))
 
-  | Ast.Nest(stmt_dots,whencode,multi,bef,aft) ->
+  | Ast.Nest(starter,stmt_dots,ender,whencode,multi,bef,aft) ->
       (* label in recursive call is None because label check is already
 	 wrapped around the corresponding code *)
 
@@ -1906,12 +1919,20 @@ and statement stmt after quantified minus_quantified
       (* no minus version because when code doesn't contain any minus code *)
       let new_quantified = Common.union_set bfvs quantified in
 
+      let dot_code =
+	match Ast.get_mcodekind starter with (*ender must have the same mcode*)
+	  Ast.MINUS(_,_,_,_) as d ->
+            (* no need for the fresh metavar, but ... is a bit weird as a
+	       variable name *)
+	    Some(make_match (make_meta_rule_elem d ([],[],[])))
+	| _ -> None in
+
       quantify guard bfvs
 	(let dots_pattern =
 	  statement_list stmt_dots (a2n after) new_quantified minus_quantified
 	    None llabel slabel true guard in
 	dots_and_nests multi
-	  (Some dots_pattern) whencode bef aft None after label
+	  (Some dots_pattern) whencode bef aft dot_code after label
 	  (process_bef_aft new_quantified minus_quantified
 	     None llabel slabel true)
 	  (function x ->
@@ -2167,12 +2188,16 @@ and statement stmt after quantified minus_quantified
 	Common.union_set mb1fvs
 	  (Common.union_set mb2fvs
 	     (Common.union_set mb3fvs minus_quantified)) in
+      let not_minus = function Ast.MINUS(_,_,_,_) -> false | _ -> true in
       let optim1 =
 	match (Ast.undots body,
 	       contains_modif rbrace or contains_pos rbrace) with
 	  ([body],false) ->
 	    (match Ast.unwrap body with
-	      Ast.Nest(stmt_dots,[],false,_,_) ->
+	      Ast.Nest(starter,stmt_dots,ender,[],false,_,_)
+		(* perhaps could optimize for minus case too... TODO *)
+		when not_minus (Ast.get_mcodekind starter)
+	      ->
             (* special case for function header + body - header is unambiguous
 	       and unique, so we can just look for the nested body anywhere
 	       else in the CFG *)
@@ -2196,9 +2221,14 @@ and statement stmt after quantified minus_quantified
 		     (match d with
 		       Ast.MINUS(_,_,_,_) -> None
 		     | _ ->
+			 let pv =
+			   (* no nested braces, because only dots *)
+			   string2var ("p1") in
+			 let paren_pred =
+			   CTL.Pred(Lib_engine.Paren pv,CTL.Control) in
 			 Some (
 			 make_seq
-			   [start_brace;
+			   [ctl_and start_brace paren_pred;
 			     match whencode with
 			       [] -> CTL.True
 			     | _ ->
@@ -2242,7 +2272,10 @@ and statement stmt after quantified minus_quantified
 						CTL.True
 					    | Ast.WhenModifier(_) -> prev)
 					CTL.True whencode) in
-				 ctl_au leftarg (make_match stripped_rbrace)]))
+				 ctl_au leftarg
+				   (ctl_and
+				      (make_match stripped_rbrace)
+				      paren_pred)]))
 	    | _ -> None)
 	| _ -> None in
       let optim2 =
