@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -55,7 +57,7 @@ let get_vars = function
     Lib_engine.Match(re) -> (Ast.get_fvs re, Ast.get_fresh re)
   | _ -> ([],[])
 
-let string2val str = Lib_engine.NormalMetaVal(Ast_c.MetaIdVal(str))
+let string2val str = Lib_engine.NormalMetaVal(Ast_c.MetaIdVal(str,[]))
 
 (* ----------------------------------------------------------------------- *)
 (* Get values for fresh variables *)
@@ -92,7 +94,7 @@ let process_tree inherited_env l =
 		     | Ast.SeedId id ->
 			 try
 			   (match List.assoc id env with
-			     Lib_engine.NormalMetaVal(Ast_c.MetaIdVal(str)) ->
+			     Lib_engine.NormalMetaVal(Ast_c.MetaIdVal(str,_)) ->
 			       str
 			   | _ -> failwith "bad id value")
 			 with
