@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -30,11 +32,16 @@ let show_trying = ref false
 
 let track_iso_usage = ref false
 
-type scanner = Glimpse | Google of string | NoScanner
-let scanner = ref NoScanner
+type scanner = Glimpse | Grep | Google of string | NoScanner
+let scanner = ref Grep
 
 let pyoutput = ref "coccilib.output.Console"
 
+let ocamlc = ref "ocamlc"
+let ocamlopt = ref "ocamlopt"
+let ocamldep = ref "ocamldep"
+let ocamlfind = ref "ocamlfind"
+
 (*"Some" value is the path with respect to which the patch should be created*)
 let patch = ref (None : string option)
 
