@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -35,10 +37,13 @@ module FC = Flag_cocci
 let cocci_file = ref ""
 
 let output_file = ref ""
-let inplace_modif = ref false  (* but keeps a .cocci_orig *)
+let inplace_modif = ref false  (* but keeps nothing *)
+let backup_suffix =
+  ref (None : string option) (* suffix for backup if one is desired *)
 let outplace_modif = ref false (* generates a .cocci_res  *)
 let preprocess = ref false     (* run the C preprocessor before cocci *)
 let compat_mode = ref false
+let ignore_unknown_opt = ref false
 
 (* somehow obsolete now *)
 let dir = ref false
@@ -268,9 +273,11 @@ let short_options = [
 
   "-o", Arg.Set_string output_file,
   "   <file> the output file";
-  "-inplace", Arg.Set inplace_modif,
+  "-in_place", Arg.Set inplace_modif,
   "   do the modification on the file directly";
-  "-outplace", Arg.Set outplace_modif,
+  "-backup_suffix", Arg.String (function s -> backup_suffix := Some s),
+  "   suffix to use when making a backup for inplace";
+  "-out_place", Arg.Set outplace_modif,
   "   store modifications in a .cocci_res file";
 
   "-U", Arg.Int (fun n -> Flag_parsing_c.diff_lines := Some (i_to_s n)),
@@ -294,6 +301,8 @@ let short_options = [
   "-local_includes",
   Arg.Unit (function _ -> FC.include_options := FC.I_NORMAL_INCLUDES),
   "  causes local include files to be used";
+  "-ignore_unknown_options", Arg.Set ignore_unknown_opt,
+  "    For integration in a toolchain (must be set before the first unknown option)";
   "-include_headers", Arg.Set include_headers,
   "    process header files independently";
   "-I",   Arg.String (fun x ->
@@ -331,7 +340,7 @@ let short_options = [
     "  guess what";
 
   "-date",   Arg.Unit (fun () ->
-    pr2 "version: $Date: 2010/03/09 08:29:04 $";
+    pr2 "version: $Date: 2010/07/01 13:22:53 $";
     raise (Common.UnixExit 0)
     ),
   "   guess what";
@@ -641,17 +650,48 @@ let all_options =
 let arg_align2 xs =
   Arg.align xs +> List.rev +> Common.drop 2 +> List.rev
 
+(*
+  Ignore unknown option
+
+  This simplifies the integration of Coccinelle in toolchain.  For
+  instance, spatch can then be used as a checker in the Linux build
+  system.
+
+*)
+let check_include_path () =
+  let opt = Array.get Sys.argv !Arg.current in
+  let is_include_re = Str.regexp "-I\\(.*\\)" in
+  if Str.string_match is_include_re opt 0 then
+    let path = Str.matched_group 1 opt in
+	FC.include_path:= path::!FC.include_path
+  else ()
+
+let rec arg_parse_no_fail l f msg =
+  try
+    check_include_path ();
+    Arg.parse_argv Sys.argv l f msg;
+  with
+    | Arg.Bad emsg ->
+	arg_parse_no_fail l f msg
+    | Arg.Help msg -> (* printf "%s" msg; exit 0; *)
+	raise Impossible  (* -help is specified in speclist *)
+
 (* copy paste of Arg.parse. Don't want the default -help msg *)
 let arg_parse2 l f msg =
   (try
     Arg.parse_argv Sys.argv l f msg;
   with
-  | Arg.Bad msg -> (* eprintf "%s" msg; exit 2; *)
-      let xs = Common.lines msg in
-      (* take only head, it's where the error msg is *)
-      pr2 (List.hd xs);
-      !short_usage_func();
-      raise (Common.UnixExit (2))
+  | Arg.Bad emsg -> (* eprintf "%s" msg; exit 2; *)
+      if not !ignore_unknown_opt then
+	begin
+	  let xs = Common.lines emsg in
+	    (* take only head, it's where the error msg is *)
+	    pr2 (List.hd xs);
+	    !short_usage_func();
+	    raise (Common.UnixExit (2))
+	end
+      else
+	arg_parse_no_fail l f msg;
   | Arg.Help msg -> (* printf "%s" msg; exit 0; *)
       raise Impossible  (* -help is specified in speclist *)
   )
@@ -700,7 +740,7 @@ let glimpse_filter (coccifile, isofile) dir =
     None -> pr2 "no glimpse keyword inferred from snippet"; None
   | Some [query] ->
       (let suffixes = if !include_headers then ["c";"h"] else ["c"] in
-      pr2 ("glimpse request = " ^ query);
+      Printf.fprintf stderr "%s\n" ("glimpse request = " ^ query);
       let command = spf "glimpse -y -H %s -N -W -w '%s'" dir query in
       let (glimpse_res,stat) = Common.cmd_to_list_and_status command in
       match stat with
@@ -741,6 +781,7 @@ let main_action xs =
 		("warning: patch output can only be created when only one\n"^
 		    "directory is specified or when the -patch flag is used")
           );
+	Flag.dir := x;
 
         let infiles =
           Common.profile_code "Main.infiles computation" (fun () ->
@@ -754,7 +795,6 @@ let main_action xs =
                 if not (null xs)
                 then failwith "-use_glimpse can accept only one dir";
 
-		Flag.dir := x;
                 let files =
 		  match glimpse_filter (!cocci_file, !Config.std_iso) x with
 		  None ->
@@ -847,6 +887,8 @@ let main_action xs =
 							   )
 		    with
 		      | Common.UnixExit x -> raise (Common.UnixExit x)
+		      |	Pycocci.Pycocciexception ->
+			  raise Pycocci.Pycocciexception
 		      | e ->
 			  (*pr2 "previous";
 			  pr2 s;
@@ -870,7 +912,10 @@ let main_action xs =
 	    outopt +> Common.do_option (fun outfile ->
 	      if !inplace_modif
 	      then begin
-                Common.command2 ("cp "^infile^" "^infile^".cocci_orig");
+		(match !backup_suffix with
+		  Some backup_suffix ->
+		    Common.command2 ("cp "^infile^" "^infile^backup_suffix)
+		| None -> ());
                 Common.command2 ("cp "^outfile^" "^infile);
 	      end;
 
@@ -936,9 +981,10 @@ let main () =
 	 then
 	   begin
 	     let chosen = List.hd !args in
-	     pr2 ("ignoring all but the last specified directory: "^chosen);
-	     args := [chosen];
-	     chosen
+	       Flag.dir := chosen;
+	       pr2 ("ignoring all but the last specified directory: "^chosen);
+	       args := [chosen];
+	       chosen
 	   end
 	 else List.hd !args in
       if !FC.include_path =*= []
@@ -975,18 +1021,20 @@ let main () =
     | [x] when !test_mode    ->
 	begin
 	  let prefix = "tests/" in
-	  try
-              FC.include_path := [prefix^"include"];
-              Testing.testone prefix x !compare_with_expected
-	  with error ->
-	    let testfile = prefix ^ x ^ ".cocci" in
-	    if not (Sys.file_exists testfile) then
+	  let testfile = x ^ ".cocci" in
+	    if Sys.file_exists (prefix ^ testfile) then
 	      begin
-		FC.include_path := ["include"];
-		Testing.testone "" x !compare_with_expected
+		FC.include_path := [prefix^"include"];
+		Testing.testone prefix x !compare_with_expected
 	      end
 	    else
-	      raise error
+	      if Sys.file_exists testfile then
+		begin
+		  FC.include_path := ["include"];
+		  Testing.testone "" x !compare_with_expected
+		end
+	      else
+		pr2 (spf "ERROR: File %s does not exist" testfile)
 	end
 
     | []  when !test_all ->
