@@ -1,11 +1,13 @@
 exception CantBeInPlus
 
 type pos = Before | After | InPlace
+type nlhint = StartBox | EndBox | SpaceOrNewline of string ref
 
 val pp_list_list_any :
   Ast_c.metavars_binding list *
   (* pr cocci *)
-  (string -> int (*line*) -> int (*lcol*) -> int (*rcol*) -> unit) *
+  (string -> int (*line*) -> int (*lcol*) -> int (*rcol*) -> nlhint option
+    -> unit) *
     (Ast_c.info -> unit) (* pr c *) *
     (unit -> unit) (* pr C space *) *
     (unit -> unit) (* pr space *) *
