@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -158,10 +160,14 @@ and expression = base_expression wrap
 
 and constraints =
     NoConstraint
-  | NotIdCstrt     of Ast.idconstraint
+  | NotIdCstrt     of Ast.reconstraint
   | NotExpCstrt    of expression list
+  | SubExpCstrt    of Ast.meta_name list
 
-and listlen = Ast.meta_name mcode option
+and listlen =
+    MetaListLen of Ast.meta_name mcode
+  | CstListLen of int
+  | AnyListLen
 
 (* --------------------------------------------------------------------- *)
 (* Types *)
@@ -397,10 +403,12 @@ and parsed_rule =
       (rule * Ast.metavar list *
 	 (string list * string list * Ast.dependency * string * Ast.exists)) *
 	(rule * Ast.metavar list) * Ast.ruletype
-  | ScriptRule of
-      string * Ast.dependency * (string * Ast.meta_name) list * string
-  | InitialScriptRule of string * Ast.dependency * string
-  | FinalScriptRule of string * Ast.dependency * string
+  | ScriptRule of string (* name *) *
+      string * Ast.dependency *
+	(Ast.script_meta_name * Ast.meta_name * Ast.metavar) list *
+	string
+  | InitialScriptRule of  string (* name *) *string * Ast.dependency * string
+  | FinalScriptRule of  string (* name *) *string * Ast.dependency * string
 
 (* --------------------------------------------------------------------- *)
 
