@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -148,10 +150,14 @@ and expression = base_expression wrap
 
 and constraints =
     NoConstraint
-  | NotIdCstrt     of Ast_cocci.idconstraint
+  | NotIdCstrt     of Ast_cocci.reconstraint
   | NotExpCstrt    of expression list
+  | SubExpCstrt    of Ast_cocci.meta_name list
 
-and listlen = Ast_cocci.meta_name mcode option
+and listlen =
+    MetaListLen of Ast_cocci.meta_name mcode
+  | CstListLen of int
+  | AnyListLen
 
 (* --------------------------------------------------------------------- *)
 (* Types *)
@@ -390,12 +396,14 @@ and parsed_rule =
 	 (string list * string list * Ast_cocci.dependency * string *
 	    Ast_cocci.exists)) *
 	(rule * Ast_cocci.metavar list) * Ast_cocci.ruletype
-  | ScriptRule of
-      string * Ast_cocci.dependency * (string * Ast_cocci.meta_name) list *
+  | ScriptRule of string (* name *) *
+      string * Ast_cocci.dependency *
+	(Ast_cocci.script_meta_name *
+	   Ast_cocci.meta_name * Ast_cocci.metavar) list *
 	string
-  | InitialScriptRule of
+  | InitialScriptRule of string (* name *) *
       string (*language*) * Ast_cocci.dependency * string (*code*)
-  | FinalScriptRule   of
+  | FinalScriptRule of string (* name *) *
       string (*language*) * Ast_cocci.dependency * string (*code*)
 
 (* --------------------------------------------------------------------- *)
