@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -78,7 +80,7 @@ val add_init_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
 val add_param_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
 
 val add_paramlist_meta:
-    (Ast_cocci.meta_name -> Ast_cocci.meta_name option -> Ast0_cocci.pure ->
+    (Ast_cocci.meta_name -> Ast_cocci.list_len -> Ast0_cocci.pure ->
       unit) ref
 
 val add_const_meta:
@@ -101,7 +103,7 @@ val add_local_idexp_meta:
       econstraints -> Ast0_cocci.pure -> unit) ref
 
 val add_explist_meta:
-    (Ast_cocci.meta_name -> Ast_cocci.meta_name option -> Ast0_cocci.pure ->
+    (Ast_cocci.meta_name -> Ast_cocci.list_len -> Ast0_cocci.pure ->
       unit) ref
 
 val add_stm_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
