@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -86,8 +88,16 @@ let collect_refs include_constraints =
   let astfvident recursor k i =
     bind (k i)
       (match Ast.unwrap i with
-	Ast.MetaId(name,_,_,_) | Ast.MetaFunc(name,_,_,_)
-      | Ast.MetaLocalFunc(name,_,_,_) -> [metaid name]
+	Ast.MetaId(name,idconstraint,_,_) | Ast.MetaFunc(name,idconstraint,_,_)
+      | Ast.MetaLocalFunc(name,idconstraint,_,_) ->
+	  let metas =
+	    if include_constraints
+	    then
+	      match idconstraint with
+		Ast.IdNegIdSet (_,metas) -> metas
+	      | _ -> []
+	    else [] in
+	  bind (List.rev metas) [metaid name]
       | _ -> option_default) in
 
   let rec type_collect res = function
@@ -101,13 +111,29 @@ let collect_refs include_constraints =
   let astfvexpr recursor k e =
     bind (k e)
       (match Ast.unwrap e with
-	Ast.MetaExpr(name,_,_,Some type_list,_,_) ->
+	Ast.MetaExpr(name,constraints,_,Some type_list,_,_) ->
 	  let types = List.fold_left type_collect option_default type_list in
-	  bind [metaid name] types
-      | Ast.MetaErr(name,_,_,_) | Ast.MetaExpr(name,_,_,_,_,_) -> [metaid name]
-      | Ast.MetaExprList(name,None,_,_) -> [metaid name]
-      | Ast.MetaExprList(name,Some (lenname,_,_),_,_) ->
+	  let extra =
+	    if include_constraints
+	    then
+	      match constraints with
+		Ast.SubExpCstrt l -> l
+	      |	_ -> []
+	    else [] in
+	  bind extra (bind [metaid name] types)
+      | Ast.MetaErr(name,constraints,_,_)
+      | Ast.MetaExpr(name,constraints,_,_,_,_) ->
+	  let extra =
+	    if include_constraints
+	    then
+	      match constraints with
+		Ast.SubExpCstrt l -> l
+	      |	_ -> []
+	    else [] in
+	  bind extra [metaid name]
+      | Ast.MetaExprList(name,Ast.MetaListLen (lenname,_,_),_,_) ->
 	  [metaid name;metaid lenname]
+      | Ast.MetaExprList(name,_,_,_) -> [metaid name]
       | Ast.DisjExpr(exps) -> bind_disj (List.map k exps)
       | _ -> option_default) in
 
@@ -139,9 +165,9 @@ let collect_refs include_constraints =
     bind (k p)
       (match Ast.unwrap p with
 	Ast.MetaParam(name,_,_) -> [metaid name]
-      | Ast.MetaParamList(name,None,_,_) -> [metaid name]
-      | Ast.MetaParamList(name,Some(lenname,_,_),_,_) ->
+      | Ast.MetaParamList(name,Ast.MetaListLen(lenname,_,_),_,_) ->
 	  [metaid name;metaid lenname]
+      | Ast.MetaParamList(name,_,_,_) -> [metaid name]
       | _ -> option_default) in
 
   let astfvrule_elem recursor k re =
@@ -221,13 +247,14 @@ let collect_saved =
       bind (k e)
 	(match Ast.unwrap e with
 	  Ast.MetaErr(name,_,TC.Saved,_) | Ast.MetaExpr(name,_,TC.Saved,_,_,_)
-	| Ast.MetaExprList(name,None,TC.Saved,_) -> [metaid name]
-	| Ast.MetaExprList(name,Some (lenname,ls,_),ns,_) ->
+	  -> [metaid name]
+	| Ast.MetaExprList(name,Ast.MetaListLen (lenname,ls,_),ns,_) ->
 	    let namesaved =
 	      match ns with TC.Saved -> [metaid name] | _ -> [] in
 	    let lensaved =
 	      match ls with TC.Saved -> [metaid lenname] | _ -> [] in
 	    lensaved @ namesaved
+	| Ast.MetaExprList(name,_,TC.Saved,_) -> [metaid name]
 	| _ -> option_default) in
     bind tymetas vars in
 
@@ -246,14 +273,14 @@ let collect_saved =
   let astfvparam recursor k p =
     bind (k p)
       (match Ast.unwrap p with
-	Ast.MetaParam(name,TC.Saved,_)
-      | Ast.MetaParamList(name,None,_,_) -> [metaid name]
-      | Ast.MetaParamList(name,Some (lenname,ls,_),ns,_) ->
+	Ast.MetaParam(name,TC.Saved,_) -> [metaid name]
+      | Ast.MetaParamList(name,Ast.MetaListLen (lenname,ls,_),ns,_) ->
 	  let namesaved =
 	    match ns with TC.Saved -> [metaid name] | _ -> [] in
 	  let lensaved =
 	    match ls with TC.Saved -> [metaid lenname] | _ -> [] in
 	  lensaved @ namesaved
+      | Ast.MetaParamList(name,_,_,_) -> [metaid name]
       | _ -> option_default) in
 
   let astfvrule_elem recursor k re =
@@ -472,19 +499,21 @@ let classify_variables metavar_decls minirules used_after =
 	let (unitary,inherited) = classify name in
 	let ty = get_option (List.map type_infos) ty in
 	Ast.rewrap e (Ast.MetaExpr(name,constraints,unitary,ty,form,inherited))
-    | Ast.MetaExprList(name,None,_,_) ->
-	(* lenname should have the same properties of being unitary or
-	   inherited as name *)
-	let (unitary,inherited) = classify name in
-	Ast.rewrap e (Ast.MetaExprList(name,None,unitary,inherited))
-    | Ast.MetaExprList(name,Some(lenname,_,_),_,_) ->
+    | Ast.MetaExprList(name,Ast.MetaListLen(lenname,_,_),_,_) ->
 	(* lenname should have the same properties of being unitary or
 	   inherited as name *)
 	let (unitary,inherited) = classify name in
 	let (lenunitary,leninherited) = classify lenname in
 	Ast.rewrap e
 	  (Ast.MetaExprList
-	     (name,Some(lenname,lenunitary,leninherited),unitary,inherited))
+	     (name,
+	      Ast.MetaListLen(lenname,lenunitary,leninherited),
+	      unitary,inherited))
+    | Ast.MetaExprList(name,lenname,_,_) ->
+	(* lenname should have the same properties of being unitary or
+	   inherited as name *)
+	let (unitary,inherited) = classify name in
+	Ast.rewrap e (Ast.MetaExprList(name,lenname,unitary,inherited))
     | _ -> e in
 
   let typeC r k e =
@@ -509,15 +538,16 @@ let classify_variables metavar_decls minirules used_after =
       Ast.MetaParam(name,_,_) ->
 	let (unitary,inherited) = classify name in
 	Ast.rewrap e (Ast.MetaParam(name,unitary,inherited))
-    | Ast.MetaParamList(name,None,_,_) ->
-	let (unitary,inherited) = classify name in
-	Ast.rewrap e (Ast.MetaParamList(name,None,unitary,inherited))
-    | Ast.MetaParamList(name,Some (lenname,_,_),_,_) ->
+    | Ast.MetaParamList(name,Ast.MetaListLen (lenname,_,_),_,_) ->
 	let (unitary,inherited) = classify name in
 	let (lenunitary,leninherited) = classify lenname in
 	Ast.rewrap e
 	  (Ast.MetaParamList
-	     (name,Some (lenname,lenunitary,leninherited),unitary,inherited))
+	     (name,Ast.MetaListLen(lenname,lenunitary,leninherited),
+	      unitary,inherited))
+    | Ast.MetaParamList(name,lenname,_,_) ->
+	let (unitary,inherited) = classify name in
+	Ast.rewrap e (Ast.MetaParamList(name,lenname,unitary,inherited))
     | _ -> e in
 
   let rule_elem r k e =
@@ -687,8 +717,8 @@ let collect_astfvs rules =
       [] -> []
     | (metavars, rule)::rules ->
         match rule with
-          Ast.ScriptRule (_,_,_,_)
-	| Ast.InitialScriptRule (_,_,_) | Ast.FinalScriptRule (_,_,_) ->
+          Ast.ScriptRule (_,_,_,_,_)
+	| Ast.InitialScriptRule (_,_,_,_) | Ast.FinalScriptRule (_,_,_,_) ->
 	    (* bound stays as is because script rules have no names, so no
 	       inheritance is possible *)
 	    rule::(loop bound rules)
@@ -765,9 +795,10 @@ let collect_top_level_used_after metavar_rule_list =
 	      used_after in
 	  let free_vars =
             match r with
-              Ast.ScriptRule (_,_,mv,_) ->
-                drop_virt(List.map (function (_,(r,v)) -> (r,v)) mv)
-            | Ast.InitialScriptRule (_,_,_) | Ast.FinalScriptRule (_,_,_) -> []
+              Ast.ScriptRule (_,_,_,mv,_) ->
+                drop_virt(List.map (function (_,(r,v),_) -> (r,v)) mv)
+            | Ast.InitialScriptRule (_,_,_,_)
+	    | Ast.FinalScriptRule (_,_,_,_) -> []
             | Ast.CocciRule (_,_,rule,_,_) ->
 		drop_virt
 	          (Common.union_set (nub (collect_all_rule_refs rule))
@@ -874,8 +905,8 @@ let collect_used_after metavar_rule_list =
     (function (metavars,r) ->
       function used_after ->
         match r with
-          Ast.ScriptRule (_,_,_,_)
-	| Ast.InitialScriptRule (_,_,_) | Ast.FinalScriptRule (_,_,_) ->
+          Ast.ScriptRule (_,_,_,_,_)
+	| Ast.InitialScriptRule (_,_,_,_) | Ast.FinalScriptRule (_,_,_,_) ->
 	    ([], [used_after], [], [])
         | Ast.CocciRule (name, rule_info, minirules, _,_) ->
           collect_local_used_after metavars minirules used_after
