@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -244,9 +246,10 @@ let rec rename_param old_name all param =
       let new_id =
 	Ast0.rewrap param
 	  (Ast0.MetaParamList(Ast0.rewrap_mcode d nm,
-			      Some (Ast0.rewrap_mcode d nml),
+			      Ast0.MetaListLen (Ast0.rewrap_mcode d nml),
 			      Ast0.Pure)) in
-      ([Ast.MetaParamListDecl(Ast.NONE,nm,Some nml);Ast.MetaListlenDecl(nml)],
+      ([Ast.MetaParamListDecl(Ast.NONE,nm,Ast.MetaLen nml);
+	 Ast.MetaListlenDecl(nml)],
        new_id)
   | Ast0.OptParam(p) ->
       let (metavars,p) = rename_param old_name all p in
