@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -37,6 +39,11 @@ module TC = Type_cocci
     constants.
 *)
 
+(* This doesn't do the . -> trick of get_constants for record fields, as
+    that does not fit well with the recursive structure.  It was not clear
+    that that was completely safe either, although eg putting a newline
+    after the . or -> is probably unusual. *)
+
 (* ----------------------------------------------------------------------- *)
 (* This phase collects everything.  One can then filter out what it not
 wanted *)
@@ -54,7 +61,10 @@ let interpret_glimpse strict x =
     | Or [x] -> loop x
     | And l -> Printf.sprintf "{%s}" (String.concat ";" (List.map loop l))
     | Or l -> Printf.sprintf "{%s}" (String.concat "," (List.map loop l))
-    | True -> "True"
+    | True ->
+	if strict
+	then failwith "True should not be in the final result"
+	else "True"
     | False ->
 	if strict
 	then failwith "False should not be in the final result.  Perhaps your rule doesn't contain any +/-/* code"
@@ -65,6 +75,26 @@ let interpret_glimpse strict x =
       failwith "False should not be in the final result.  Perhaps your rule doesn't contain any +/-/* code"
   | _ -> Some [(loop x)]
 
+(* grep only does or *)
+let interpret_grep strict x =
+  let rec loop = function
+      Elem x -> [x]
+    | And l -> List.concat (List.map loop l)
+    | Or l -> List.concat (List.map loop l)
+    | True ->
+	if strict
+	then failwith "True should not be in the final result"
+	else ["True"]
+    | False ->
+	if strict
+	then failwith "False should not be in the final result.  Perhaps your rule doesn't contain any +/-/* code"
+	else ["False"] in
+  match x with
+    True -> None
+  | False when strict ->
+      failwith "False should not be in the final result.  Perhaps your rule doesn't contain any +/-/* code"
+  | _ -> Some (loop x)
+
 let interpret_google strict x =
   (* convert to dnf *)
   let rec dnf = function
@@ -92,14 +122,8 @@ let interpret_google strict x =
       failwith "False should not be in the final result.  Perhaps your rule doesn't contain any +/-/* code"
   | _ -> Some (dnf x)
 
-let interpret strict x =
-  match !Flag.scanner with
-    Flag.Glimpse -> interpret_glimpse strict x
-  | Flag.Google _ -> interpret_google strict x
-  | _ -> failwith "not possible"
-
 let combine2c x =
-  match interpret false x with
+  match interpret_glimpse false x with
     None -> "None"
   | Some x -> String.concat " || " x
 
@@ -244,13 +268,13 @@ let do_get_constants constants keywords env neg_pos =
 	bind (k e) (bind (minherited name) types)
     | Ast.MetaErr(name,_,_,_) | Ast.MetaExpr(name,_,_,_,_,_) ->
 	bind (k e) (minherited name)
-    | Ast.MetaExprList(name,None,_,_) -> minherited name
-    | Ast.MetaExprList(name,Some (lenname,_,_),_,_) ->
+    | Ast.MetaExprList(name,Ast.MetaListLen (lenname,_,_),_,_) ->
 	bind (k e) (bind (minherited name) (minherited lenname))
+    | Ast.MetaExprList(name,_,_,_) -> minherited name
     | Ast.SizeOfExpr(sizeof,exp) -> bind (keywords "sizeof") (k e)
     | Ast.SizeOfType(sizeof,lp,ty,rp) -> bind (keywords "sizeof") (k e)
-    | Ast.NestExpr(expr_dots,wc,false) -> option_default
-    | Ast.NestExpr(expr_dots,wc,true) ->
+    | Ast.NestExpr(starter,expr_dots,ender,wc,false) -> option_default
+    | Ast.NestExpr(starter,expr_dots,ender,wc,true) ->
 	r.V.combiner_expression_dots expr_dots
     | Ast.DisjExpr(exps) ->
 	disj_union_all (List.map r.V.combiner_expression exps)
@@ -298,9 +322,9 @@ let do_get_constants constants keywords env neg_pos =
     match Ast.unwrap p with
       Ast.OptParam(param) -> option_default
     | Ast.MetaParam(name,_,_) -> bind (k p) (minherited name)
-    | Ast.MetaParamList(name,None,_,_) -> bind (k p) (minherited name)
-    | Ast.MetaParamList(name,Some(lenname,_,_),_,_) ->
+    | Ast.MetaParamList(name,Ast.MetaListLen(lenname,_,_),_,_) ->
 	bind (minherited name) (bind (minherited lenname) (k p))
+    | Ast.MetaParamList(name,_,_,_) -> bind (k p) (minherited name)
     | _ -> k p in
 
   let rule_elem r k re =
@@ -347,8 +371,8 @@ let do_get_constants constants keywords env neg_pos =
     match Ast.unwrap s with
       Ast.Disj(stmt_dots) ->
 	disj_union_all (List.map r.V.combiner_statement_dots stmt_dots)
-    | Ast.Nest(stmt_dots,whn,false,_,_) -> option_default
-    | Ast.Nest(stmt_dots,whn,true,_,_) ->
+    | Ast.Nest(starter,stmt_dots,ender,whn,false,_,_) -> option_default
+    | Ast.Nest(starter,stmt_dots,ender,whn,true,_,_) ->
 	r.V.combiner_statement_dots stmt_dots
     | Ast.OptStm(s) -> option_default
     | Ast.Dots(d,whn,_,_) | Ast.Circles(d,whn,_,_) | Ast.Stars(d,whn,_,_) ->
@@ -400,16 +424,15 @@ let get_plus_constants =
   let donothing r k e = k e in
   let bind = Common.union_set in
   let option_default = [] in
-  let mcode r mc =
-    let mcodekind = Ast.get_mcodekind mc in
-    let recurse l =
-      List.fold_left
-	(List.fold_left
-	   (function prev ->
-	     function cur ->
-	       bind ((get_all_constants false).V.combiner_anything cur) prev))
-	[] l in
-    match mcodekind with
+
+  let recurse l =
+    List.fold_left
+      (List.fold_left
+	 (function prev ->
+	   function cur ->
+	     bind ((get_all_constants false).V.combiner_anything cur) prev))
+      [] l in
+  let process_mcodekind = function
       Ast.MINUS(_,_,_,anythings) -> recurse anythings
     | Ast.CONTEXT(_,Ast.BEFORE(a,_)) -> recurse a
     | Ast.CONTEXT(_,Ast.AFTER(a,_)) -> recurse a
@@ -417,11 +440,27 @@ let get_plus_constants =
 	Common.union_set (recurse a1) (recurse a2)
     | _ -> [] in
 
+  let mcode r mc = process_mcodekind (Ast.get_mcodekind mc) in
+  let end_info (_,_,_,mc) = process_mcodekind mc in
+
+  let rule_elem r k e =
+    match Ast.unwrap e with
+      Ast.FunHeader(bef,_,_,_,_,_,_)
+    | Ast.Decl(bef,_,_) -> bind (process_mcodekind bef) (k e)
+    | _ -> k e in
+
+  let statement r k e =
+    match Ast.unwrap e with
+      Ast.IfThen(_,_,ei) | Ast.IfThenElse(_,_,_,_,ei)
+    | Ast.While(_,_,ei)  | Ast.For(_,_,ei)
+    | Ast.Iterator(_,_,ei) -> bind (k e) (end_info ei)
+    | _ -> k e in
+
   V.combiner bind option_default
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
     donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing
+    rule_elem statement donothing donothing donothing
 
 (* ------------------------------------------------------------------------ *)
 
@@ -444,16 +483,38 @@ let all_context =
 
   let donothing recursor k e = k e in
 
-  let mcode r e =
-    match Ast.get_mcodekind e with
+  let process_mcodekind = function
       Ast.CONTEXT(_,Ast.NOTHING) -> true
     | _ -> false in
 
+  let mcode r e = process_mcodekind (Ast.get_mcodekind e) in
+
+  let end_info (_,_,_,mc) = process_mcodekind mc in
+
+  let initialiser r k e =
+    match Ast.unwrap e with
+      Ast.InitList(all_minus,_,_,_,_) ->
+	not all_minus && k e
+    | _ -> k e in
+
+  let rule_elem r k e =
+    match Ast.unwrap e with
+      Ast.FunHeader(bef,_,_,_,_,_,_)
+    | Ast.Decl(bef,_,_) -> bind (process_mcodekind bef) (k e)
+    | _ -> k e in
+
+  let statement r k e =
+    match Ast.unwrap e with
+      Ast.IfThen(_,_,ei) | Ast.IfThenElse(_,_,_,_,ei)
+    | Ast.While(_,_,ei)  | Ast.For(_,_,ei)
+    | Ast.Iterator(_,_,ei) -> bind (k e) (end_info ei)
+    | _ -> k e in
+
   V.combiner bind option_default
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
     donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing
+    donothing donothing donothing donothing initialiser donothing
+    donothing rule_elem statement donothing donothing donothing
 
 (* ------------------------------------------------------------------------ *)
 
@@ -491,46 +552,57 @@ let rule_fn tls in_plus env neg_pos =
 	| x -> (build_or x rest_info, new_plusses))
     (False,in_plus) (List.combine tls neg_pos)
 
-let get_constants rules neg_pos_vars =
-  match !Flag.scanner with
-    Flag.NoScanner -> None
-  | Flag.Glimpse | Flag.Google _ ->
-      let (info,_,_,_) =
-	List.fold_left
-	  (function (rest_info,in_plus,env,locals(*dom of env*)) ->
-            function
-		(Ast.ScriptRule (_,deps,mv,_),_) ->
-		  let extra_deps =
-		    List.fold_left
-		      (function prev ->
-			function (_,(rule,_)) ->
-			  if rule = "virtual"
-			  then prev
-			  else Ast.AndDep (Ast.Dep rule,prev))
-		      deps mv in
-		  (match dependencies env extra_deps with
-		    False -> (rest_info, in_plus, env, locals)
-		  | dependencies ->
-		      (build_or dependencies rest_info, in_plus, env, locals))
-              | (Ast.InitialScriptRule (_,deps,_),_)
-	      | (Ast.FinalScriptRule (_,deps,_),_) ->
+let run rules neg_pos_vars =
+  let (info,_,_,_) =
+    List.fold_left
+      (function (rest_info,in_plus,env,locals(*dom of env*)) ->
+        function
+	    (Ast.ScriptRule (_,_,deps,mv,_),_) ->
+	      let extra_deps =
+		List.fold_left
+		  (function prev ->
+		    function (_,(rule,_),_) ->
+		      if rule = "virtual"
+		      then prev
+		      else Ast.AndDep (Ast.Dep rule,prev))
+		  deps mv in
+	      (match dependencies env extra_deps with
+		False -> (rest_info, in_plus, env, locals)
+	      | dependencies ->
+		  (build_or dependencies rest_info, in_plus, env, locals))
+          | (Ast.InitialScriptRule (_,_,deps,_),_)
+	  | (Ast.FinalScriptRule (_,_,deps,_),_) ->
 		  (* initialize and finalize dependencies are irrelevant to
 		     get_constants *)
-		  (rest_info, in_plus, env, locals)
-              | (Ast.CocciRule (nm,(dep,_,_),cur,_,_),neg_pos_vars) ->
-		  let (cur_info,cur_plus) =
-		    rule_fn cur in_plus ((nm,True)::env)
-		      neg_pos_vars in
+	      (rest_info, in_plus, env, locals)
+          | (Ast.CocciRule (nm,(dep,_,_),cur,_,_),neg_pos_vars) ->
+	      let (cur_info,cur_plus) =
+		rule_fn cur in_plus ((nm,True)::env)
+		  neg_pos_vars in
+	      (match dependencies env dep with
+		False -> (rest_info,cur_plus,env,locals)
+	      | dependencies ->
 		  if List.for_all all_context.V.combiner_top_level cur
 		  then (rest_info,cur_plus,(nm,cur_info)::env,nm::locals)
 		  else
 	       (* no constants if dependent on another rule; then we need to
 	          find the constants of that rule *)
-		    match dependencies env dep with
-		      False -> (rest_info,cur_plus,env,locals)
-		    | dependencies ->
-			(build_or (build_and dependencies cur_info) rest_info,
-			 cur_plus,env,locals))
-	  (False,[],[],[])
-	  (List.combine (rules : Ast.rule list) neg_pos_vars) in
-      interpret true info
+		    (build_or (build_and dependencies cur_info) rest_info,
+		     cur_plus,env,locals)))
+      (False,[],[],[])
+      (List.combine (rules : Ast.rule list) neg_pos_vars) in
+  info
+    
+let get_constants rules neg_pos_vars =
+  match !Flag.scanner with
+    Flag.NoScanner -> (None,None)
+  | Flag.Grep ->
+      let res = run rules neg_pos_vars in
+      (interpret_grep true res,None)
+  | Flag.Glimpse ->
+      let res = run rules neg_pos_vars in
+      (interpret_grep true res,interpret_glimpse true res)
+  | Flag.Google _ ->
+      let res = run rules neg_pos_vars in
+      (interpret_grep true res,interpret_google true res)
+      
