@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -29,7 +31,7 @@ type info = Ast.meta_name * Ast0.pure * Data.clt
 type idinfo = Ast.meta_name * Data.iconstraints * Ast0.pure * Data.clt
 type expinfo = Ast.meta_name * Data.econstraints * Ast0.pure * Data.clt
 type tyinfo = Ast.meta_name * Ast0.typeC list * Ast0.pure * Data.clt
-type list_info = Ast.meta_name * Ast.meta_name option * Ast0.pure * Data.clt
+type list_info = Ast.meta_name * Ast.list_len * Ast0.pure * Data.clt
 type typed_expinfo =
     Ast.meta_name * Data.econstraints * Ast0.pure *
       Type_cocci.typeC list option * Data.clt
@@ -188,17 +190,16 @@ let iso_adjust first_fn fn first rest =
     front::after -> (first_fn first::front)::after
   | _ -> failwith "not possible"
 
-let check_meta tok =
-  let lookup rule name =
-    try
-      let info = Hashtbl.find Data.all_metadecls rule in
-      List.find (function mv -> Ast.get_meta_name mv = (rule,name)) info
-    with
-      Not_found ->
-	raise
-	  (Semantic_cocci.Semantic
-	     ("bad rule "^rule^" or bad variable "^name)) in
-  match tok with
+let lookup rule name =
+  try
+    let info = Hashtbl.find Data.all_metadecls rule in
+    List.find (function mv -> Ast.get_meta_name mv = (rule,name)) info
+  with
+    Not_found ->
+      raise
+	(Semantic_cocci.Semantic("bad rule "^rule^" or bad variable "^name))
+
+let check_meta_tyopt type_irrelevant = function
     Ast.MetaIdDecl(Ast.NONE,(rule,name)) ->
       (match lookup rule name with
 	Ast.MetaIdDecl(_,_) | Ast.MetaFreshIdDecl(_,_) -> ()
@@ -254,21 +255,21 @@ let check_meta tok =
 	       ("incompatible inheritance declaration "^name)))
   | Ast.MetaExpDecl(Ast.NONE,(rule,name),ty) ->
       (match lookup rule name with
-	Ast.MetaExpDecl(_,_,ty1) when ty = ty1 -> ()
+	Ast.MetaExpDecl(_,_,ty1) when type_irrelevant or ty = ty1 -> ()
       | _ ->
 	  raise
 	    (Semantic_cocci.Semantic
 	       ("incompatible inheritance declaration "^name)))
   | Ast.MetaIdExpDecl(Ast.NONE,(rule,name),ty) ->
       (match lookup rule name with
-	Ast.MetaIdExpDecl(_,_,ty1) when ty = ty1 -> ()
+	Ast.MetaIdExpDecl(_,_,ty1) when type_irrelevant or ty = ty1 -> ()
       | _ ->
 	  raise
 	    (Semantic_cocci.Semantic
 	       ("incompatible inheritance declaration "^name)))
   | Ast.MetaLocalIdExpDecl(Ast.NONE,(rule,name),ty) ->
       (match lookup rule name with
-	Ast.MetaLocalIdExpDecl(_,_,ty1) when ty = ty1 -> ()
+	Ast.MetaLocalIdExpDecl(_,_,ty1) when type_irrelevant or ty = ty1 -> ()
       | _ ->
 	  raise
 	    (Semantic_cocci.Semantic
@@ -311,7 +312,7 @@ let check_meta tok =
 	       ("incompatible inheritance declaration "^name)))
   | Ast.MetaConstDecl(Ast.NONE,(rule,name),ty) ->
       (match lookup rule name with
-	Ast.MetaConstDecl(_,_,ty1) when ty = ty1 -> ()
+	Ast.MetaConstDecl(_,_,ty1) when type_irrelevant or ty = ty1 -> ()
       | _ ->
 	  raise
 	    (Semantic_cocci.Semantic
@@ -332,6 +333,16 @@ let check_meta tok =
       raise
 	(Semantic_cocci.Semantic ("arity not allowed on imported declaration"))
 
+let check_meta m = check_meta_tyopt false m
+
+let check_inherited_constraint meta_name fn =
+  match meta_name with
+    (None,_) -> failwith "constraint must be an inherited variable"
+  | (Some rule,name) ->
+      let i = (rule,name) in
+      check_meta_tyopt true (fn i);
+      i
+
 let create_metadec ar ispure kindfn ids current_rule =
   List.concat
     (List.map
@@ -394,15 +405,20 @@ let create_metadec_ty ar ispure kindfn ids current_rule =
        ids)
 
 let create_len_metadec ar ispure kindfn lenid ids current_rule =
-  let lendec =
-    create_metadec Ast.NONE Ast0.Impure
-      (fun _ name _ check_meta -> check_meta(Ast.MetaListlenDecl(name)))
-      [lenid] current_rule in
-  let lenname =
-    match lendec with
-      [Common.Left (Ast.MetaListlenDecl(x))] -> x
-    | [Common.Right (Ast.MetaListlenDecl(x))] -> x
-    | _ -> failwith "unexpected length declaration" in
+  let (lendec,lenname) =
+    match lenid with
+      Common.Left lenid ->
+	let lendec =
+	  create_metadec Ast.NONE Ast0.Impure
+	    (fun _ name _ check_meta -> check_meta(Ast.MetaListlenDecl(name)))
+	    [lenid] current_rule in
+	let lenname =
+	  match lendec with
+	    [Common.Left (Ast.MetaListlenDecl(x))] -> Ast.MetaLen x
+	  | [Common.Right (Ast.MetaListlenDecl(x))] -> Ast.MetaLen x
+	  | _ -> failwith "unexpected length declaration" in
+	(lendec,lenname)
+    | Common.Right n -> ([],Ast.CstLen n) in
   lendec@(create_metadec ar ispure (kindfn lenname) ids current_rule)
 
 (* ---------------------------------------------------------------------- *)
@@ -514,15 +530,15 @@ let make_generated_rule_name_result nm d i a e ee =
 
 let make_script_rule_name_result lang deps =
   let l = id2name lang in
-  Ast.ScriptRulename (l,deps)
+  Ast.ScriptRulename (None,l,deps)
 
 let make_initial_script_rule_name_result lang deps =
   let l = id2name lang in
-  Ast.InitialScriptRulename(l,deps)
+  Ast.InitialScriptRulename(None,l,deps)
 
 let make_final_script_rule_name_result lang deps =
   let l = id2name lang in
-  Ast.FinalScriptRulename(l,deps)
+  Ast.FinalScriptRulename(None,l,deps)
 
 (* Allows type alone only when it is void and only when there is only one
     parameter.  This avoids ambiguity problems in the parser. *)
