@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -157,6 +159,7 @@ let token2c (tok,_) =
   | PC.TAnd (clt) -> "&"^(line_type2c clt)
   | PC.TEqEq(clt) -> "=="^(line_type2c clt)
   | PC.TNotEq(clt) -> "!="^(line_type2c clt)
+  | PC.TSub(clt) -> "<="^(line_type2c clt)
   | PC.TTildeEq(clt) -> "~="^(line_type2c clt)
   | PC.TTildeExclEq(clt) -> "~!="^(line_type2c clt)
   | PC.TLogOp(op,clt) ->
@@ -271,6 +274,7 @@ let token2c (tok,_) =
   | PC.TIsoStatement -> "Statement"
   | PC.TIsoDeclaration -> "Declaration"
   | PC.TIsoType -> "Type"
+  | PC.TUnderscore -> "_"
   | PC.TScriptData s -> s
 
 let print_tokens s tokens =
@@ -306,7 +310,8 @@ let plus_attachable only_plus (tok,_) =
   | PC.TString(_,clt) | PC.TChar(_,clt) | PC.TFloat(_,clt) | PC.TInt(_,clt)
 
   | PC.TOrLog(clt) | PC.TAndLog(clt) | PC.TOr(clt) | PC.TXor(clt)
-  | PC.TAnd (clt) | PC.TEqEq(clt) | PC.TNotEq(clt) | PC.TTildeEq(clt) | PC.TLogOp(_,clt)
+  | PC.TAnd (clt) | PC.TEqEq(clt) | PC.TNotEq(clt) | PC.TTildeEq(clt)
+  | PC.TLogOp(_,clt)
   | PC.TShOp(_,clt) | PC.TPlus(clt) | PC.TMinus(clt) | PC.TMul(clt)
   | PC.TDmOp(_,clt) | PC.TTilde (clt)
 
@@ -323,6 +328,9 @@ let plus_attachable only_plus (tok,_) =
   | PC.TWhen(clt) |  PC.TWhenTrue(clt) |  PC.TWhenFalse(clt)
   | PC.TAny(clt) | PC.TStrict(clt) | PC.TEllipsis(clt)
   (* | PC.TCircles(clt) | PC.TStars(clt) *)
+  | PC.TOEllipsis(clt) | PC.TCEllipsis(clt)
+  | PC.TPOEllipsis(clt) | PC.TPCEllipsis(clt) (* | PC.TOCircles(clt)
+  | PC.TCCircles(clt) | PC.TOStars(clt) | PC.TCStars(clt) *)
 
   | PC.TWhy(clt) | PC.TDotDot(clt) | PC.TBang(clt) | PC.TOPar(clt)
   | PC.TCPar(clt)
@@ -339,11 +347,9 @@ let plus_attachable only_plus (tok,_) =
       else if only_plus then NOTPLUS
       else if line_type clt = D.CONTEXT then PLUS else NOTPLUS
 
-  | PC.TOPar0(clt) | PC.TMid0(clt) | PC.TCPar0(clt)
-  | PC.TOEllipsis(clt) | PC.TCEllipsis(clt)
-  | PC.TPOEllipsis(clt) | PC.TPCEllipsis(clt) (* | PC.TOCircles(clt)
-  | PC.TCCircles(clt) | PC.TOStars(clt) | PC.TCStars(clt) *) -> NOTPLUS
+  | PC.TOPar0(clt) | PC.TMid0(clt) | PC.TCPar0(clt) -> NOTPLUS
   | PC.TMetaPos(nm,_,_,_) -> NOTPLUS
+  | PC.TSub(clt) -> NOTPLUS
 
   | _ -> SKIP
 
@@ -371,7 +377,8 @@ let get_clt (tok,_) =
   | PC.TString(_,clt) | PC.TChar(_,clt) | PC.TFloat(_,clt) | PC.TInt(_,clt)
 
   | PC.TOrLog(clt) | PC.TAndLog(clt) | PC.TOr(clt) | PC.TXor(clt)
-  | PC.TAnd (clt) | PC.TEqEq(clt) | PC.TNotEq(clt) | PC.TTildeEq(clt) | PC.TLogOp(_,clt)
+  | PC.TAnd (clt) | PC.TEqEq(clt) | PC.TNotEq(clt) | PC.TTildeEq(clt)
+  | PC.TSub(clt) | PC.TLogOp(_,clt)
   | PC.TShOp(_,clt) | PC.TPlus(clt) | PC.TMinus(clt) | PC.TMul(clt)
   | PC.TDmOp(_,clt) | PC.TTilde (clt)
 
@@ -473,6 +480,7 @@ let update_clt (tok,x) clt =
   | PC.TEqEq(_) -> (PC.TEqEq(clt),x)
   | PC.TNotEq(_) -> (PC.TNotEq(clt),x)
   | PC.TTildeEq(_) -> (PC.TTildeEq(clt),x)
+  | PC.TSub(_) -> (PC.TSub(clt),x)
   | PC.TLogOp(op,_) -> (PC.TLogOp(op,clt),x)
   | PC.TShOp(op,_) -> (PC.TShOp(op,clt),x)
   | PC.TPlus(_) -> (PC.TPlus(clt),x)
@@ -644,14 +652,15 @@ let split_token ((tok,_) as t) =
   | PC.TFunDecl(clt)
   | PC.TWhen(clt) | PC.TWhenTrue(clt) | PC.TWhenFalse(clt)
   | PC.TAny(clt) | PC.TStrict(clt) | PC.TLineEnd(clt)
-  | PC.TEllipsis(clt) (* | PC.TCircles(clt) | PC.TStars(clt) *) -> split t clt
+  | PC.TEllipsis(clt) (* | PC.TCircles(clt) | PC.TStars(clt) *)
+  | PC.TOEllipsis(clt) | PC.TCEllipsis(clt)
+  | PC.TPOEllipsis(clt) | PC.TPCEllipsis(clt) -> split t clt
 
-  | PC.TOEllipsis(_) | PC.TCEllipsis(_) (* clt must be context *)
-  | PC.TPOEllipsis(_) | PC.TPCEllipsis(_) (* clt must be context *)
 (*
   | PC.TOCircles(_) | PC.TCCircles(_)   (* clt must be context *)
   | PC.TOStars(_) | PC.TCStars(_)       (* clt must be context *)
 *)
+
   | PC.TBang0 | PC.TPlus0 | PC.TWhy0 ->
       ([t],[t])
 
@@ -665,7 +674,8 @@ let split_token ((tok,_) as t) =
       split t clt
 
   | PC.TOrLog(clt) | PC.TAndLog(clt) | PC.TOr(clt) | PC.TXor(clt)
-  | PC.TAnd (clt) | PC.TEqEq(clt) | PC.TNotEq(clt) | PC.TTildeEq(clt) | PC.TTildeExclEq(clt) | PC.TLogOp(_,clt)
+  | PC.TAnd (clt) | PC.TEqEq(clt) | PC.TNotEq(clt) | PC.TTildeEq(clt)
+  | PC.TTildeExclEq(clt) | PC.TSub(clt) | PC.TLogOp(_,clt)
   | PC.TShOp(_,clt) | PC.TPlus(clt) | PC.TMinus(clt) | PC.TMul(clt)
   | PC.TDmOp(_,clt) | PC.TTilde (clt) -> split t clt
 
@@ -677,7 +687,7 @@ let split_token ((tok,_) as t) =
   | PC.TEq(clt) | PC.TAssign(_,clt) | PC.TDot(clt) | PC.TComma(clt)
   | PC.TPtVirg(clt) -> split t clt
 
-  | PC.EOF | PC.TInvalid -> ([t],[t])
+  | PC.EOF | PC.TInvalid | PC.TUnderscore -> ([t],[t])
 
   | PC.TIso | PC.TRightIso
   | PC.TIsoExpression | PC.TIsoStatement | PC.TIsoDeclaration | PC.TIsoType
@@ -946,6 +956,36 @@ let rec translate_when_true_false = function
 
 (* ----------------------------------------------------------------------- *)
 
+(* In a nest, if the nest is -, all of the nested code must also be -.
+All are converted to context, because the next takes care of the -. *)
+let check_nests tokens =
+  let is_minus t =
+    let (line_type,a,b,c,d,e,f,g) = get_clt t in
+    List.mem line_type [D.MINUS;D.OPTMINUS;D.UNIQUEMINUS] in
+  let drop_minus t =
+    let clt = try Some(get_clt t) with Failure _ -> None in
+	match clt with
+	  Some (line_type,a,b,c,d,e,f,g) ->
+	    (match line_type with
+	      D.MINUS -> update_clt t (D.CONTEXT,a,b,c,d,e,f,g)
+	    | D.OPTMINUS -> update_clt t (D.OPT,a,b,c,d,e,f,g)
+	    | D.UNIQUEMINUS -> update_clt t (D.UNIQUE,a,b,c,d,e,f,g)
+	    | _ -> failwith "minus token expected")
+	| None -> t in
+  let rec outside = function
+      [] -> []
+    | ((PC.TPOEllipsis(clt),q) as t)::r when is_minus t -> t :: inside 0 r
+    | t::r -> t :: outside r
+  and inside stack = function
+      [] -> failwith "missing nest end"
+    | ((PC.TPCEllipsis(clt),q) as t)::r ->
+	(drop_minus t)
+	:: (if stack = 0 then outside r else inside (stack - 1) r)
+    | ((PC.TPOEllipsis(clt),q) as t)::r ->
+	(drop_minus t) :: (inside (stack + 1) r)
+    | t :: r -> (drop_minus t) :: (inside stack r) in
+  outside tokens
+
 let check_parentheses tokens =
   let clt2line (_,line,_,_,_,_,_,_) = line in
   let rec loop seen_open = function
@@ -1236,7 +1276,10 @@ let prepare_tokens tokens =
     (translate_when_true_false (* after insert_line_end *)
        (insert_line_end
 	  (detect_types false
-	     (find_function_names (detect_attr (check_parentheses tokens))))))
+	     (find_function_names
+		(detect_attr
+		   (check_nests
+		      (check_parentheses tokens)))))))
 
 let prepare_mv_tokens tokens =
   detect_types false (detect_attr tokens)
@@ -1336,9 +1379,12 @@ let get_rule_name parse_fn starts_with_name get_tokens file prefix =
           Ast.CocciRulename (check_name nm,a,b,c,d,e)
       | Ast.GeneratedRulename (nm,a,b,c,d,e) ->
           Ast.GeneratedRulename (check_name nm,a,b,c,d,e)
-      | Ast.ScriptRulename(s,deps) -> Ast.ScriptRulename(s,deps)
-      | Ast.InitialScriptRulename(s,deps) -> Ast.InitialScriptRulename(s,deps)
-      | Ast.FinalScriptRulename(s,deps) -> Ast.FinalScriptRulename(s,deps)
+      | Ast.ScriptRulename(_,s,deps) ->
+	  Ast.ScriptRulename(check_name None,s,deps)
+      | Ast.InitialScriptRulename(_,s,deps) ->
+	  Ast.InitialScriptRulename(check_name None,s,deps)
+      | Ast.FinalScriptRulename(_,s,deps) ->
+	  Ast.FinalScriptRulename(check_name None,s,deps)
     else
       Ast.CocciRulename(Some(mknm()),Ast.NoDep,[],[],Ast.Undetermined,false) in
   Data.in_rule_name := false;
@@ -1384,7 +1430,7 @@ let parse_iso file =
 	    let tokens = prepare_tokens (start@tokens) in
             (*
 	       print_tokens "iso tokens" tokens;
-	    *)
+	    å*)
 	    let entry = parse_one "iso main" PC.iso_main file tokens in
 	    let entry = List.map (List.map Test_exps.process_anything) entry in
 	    if more
@@ -1462,6 +1508,7 @@ let eval_depend dep virt =
 let parse file =
   Lexer_cocci.init();
   let rec parse_loop file =
+  Lexer_cocci.include_init ();
   let table = Common.full_charpos_to_pos file in
   Common.with_open_infile file (fun channel ->
   let lexbuf = Lexing.from_channel channel in
@@ -1489,12 +1536,12 @@ let parse file =
 	  List.iter (function x -> Hashtbl.add Lexer_cocci.rule_names x ())
 	    virt;
 
-	  let (extra_iso_files, extra_rules, extra_virt) =
+	  let (extra_iso_files, extra_rules, extra_virt, extra_metas) =
 	    let rec loop = function
-		[] -> ([],[],[])
-	      |	(a,b,c)::rest ->
-		  let (x,y,z) = loop rest in
-		  (a::x,b::y,c::z) in
+		[] -> ([],[],[],[])
+	      |	(a,b,c,d)::rest ->
+		  let (x,y,z,zz) = loop rest in
+		  (a::x,b::y,c::z,d@zz) in
 	    loop (List.map parse_loop include_files) in
 
           let parse_cocci_rule ruletype old_metas
@@ -1586,7 +1633,7 @@ let parse file =
 		  toks;
 		failwith "Malformed script rule" in
 
-          let parse_script_rule language old_metas deps =
+          let parse_script_rule name language old_metas deps =
             let get_tokens = tokens_script_all table file false lexbuf in
 
               (* meta-variables *)
@@ -1594,7 +1641,7 @@ let parse file =
 	      Data.call_in_meta
 		(function _ ->
 		  get_script_metavars PC.script_meta_main table file lexbuf) in
-
+(*
             let exists_in old_metas (py,(r,m)) =
 	      r = "virtual" or
               let test (rr,mr) x =
@@ -1611,34 +1658,35 @@ let parse file =
 		       "Script references unknown meta-variable: %s"
 		       (meta2c(snd x))))
 	      metavars;
-
+*)
               (* script code *)
             let (more, tokens) = get_tokens [PC.TArobArob; PC.TArob] in
             let data = collect_script_tokens tokens in
-            (more,Ast0.ScriptRule(language, deps, metavars, data),[],tokens) in
+            (more,Ast0.ScriptRule(name, language, deps, metavars, data),
+	     [],tokens) in
 
-          let parse_if_script_rule k language _ deps =
+          let parse_if_script_rule k name language _ deps =
             let get_tokens = tokens_script_all table file false lexbuf in
 
               (* script code *)
             let (more, tokens) = get_tokens [PC.TArobArob; PC.TArob] in
             let data = collect_script_tokens tokens in
-            (more,k (language, deps, data),[],tokens) in
+            (more,k (name, language, deps, data),[],tokens) in
 
 	  let parse_iscript_rule =
 	    parse_if_script_rule
-	      (function (language,deps,data) ->
-		Ast0.InitialScriptRule(language,deps,data)) in
+	      (function (name,language,deps,data) ->
+		Ast0.InitialScriptRule(name,language,deps,data)) in
 
 	  let parse_fscript_rule =
 	    parse_if_script_rule
-	      (function (language,deps,data) ->
-		Ast0.FinalScriptRule(language,deps,data)) in
+	      (function (name,language,deps,data) ->
+		Ast0.FinalScriptRule(name,language,deps,data)) in
 
-	  let do_parse_script_rule fn l old_metas deps =
+	  let do_parse_script_rule fn name l old_metas deps =
 	    match eval_depend deps virt with
-	      Some deps -> fn l old_metas deps
-	    | None ->  fn l old_metas Ast.FailDep in
+	      Some deps -> fn name l old_metas deps
+	    | None ->  fn name l old_metas Ast.FailDep in
 
           let parse_rule old_metas starts_with_name =
             let rulename =
@@ -1661,24 +1709,25 @@ let parse file =
 		  Some (dep) ->
 		    Data.in_generating := true;
 		    let res =
-		      parse_cocci_rule Ast.Normal old_metas (s,dep,b,c,d,e) in
+		      parse_cocci_rule Ast.Generated old_metas
+			(s,dep,b,c,d,e) in
 		    Data.in_generating := false;
 		    res
 		| None ->
 		    D.ignore_patch_or_match := true;
 		    Data.in_generating := true;
                     let res =
-		      parse_cocci_rule Ast.Normal old_metas
+		      parse_cocci_rule Ast.Generated old_metas
 			(s, Ast.FailDep, b, c, d, e) in
 		    D.ignore_patch_or_match := false;
 		    Data.in_generating := false;
 		    res)
-            | Ast.ScriptRulename(l,deps) ->
-		do_parse_script_rule parse_script_rule l old_metas deps
-            | Ast.InitialScriptRulename(l,deps) ->
-		do_parse_script_rule parse_iscript_rule l old_metas deps
-            | Ast.FinalScriptRulename(l,deps)   ->
-		do_parse_script_rule parse_fscript_rule l old_metas deps
+            | Ast.ScriptRulename(Some s,l,deps) ->
+		do_parse_script_rule parse_script_rule s l old_metas deps
+            | Ast.InitialScriptRulename(Some s,l,deps) ->
+		do_parse_script_rule parse_iscript_rule s l old_metas deps
+            | Ast.FinalScriptRulename(Some s,l,deps)   ->
+		do_parse_script_rule parse_fscript_rule s l old_metas deps
             | _ -> failwith "Malformed rule name" in
 
 	  let rec loop old_metas starts_with_name =
@@ -1694,21 +1743,28 @@ let parse file =
 
             let (more, rule, metavars, tokens) =
               parse_rule old_metas starts_with_name in
+	    let all_metas = metavars @ old_metas in
+
             if more then
-              rule::
-	      (loop (metavars @ old_metas) (gen_starts_with_name more tokens))
-            else [rule] in
+	      let (all_rules,all_metas) =
+		loop all_metas (gen_starts_with_name more tokens) in
+	      (rule::all_rules,all_metas)
+            else ([rule],all_metas) in
+
+	  let (all_rules,all_metas) =
+	    loop extra_metas (x = PC.TArob) in
 
 	  (List.fold_left
 	     (function prev -> function cur -> Common.union_set cur prev)
 	     iso_files extra_iso_files,
 	   (* included rules first *)
 	   List.fold_left (function prev -> function cur -> cur@prev)
-	     (loop [] (x = PC.TArob)) (List.rev extra_rules),
-	   List.fold_left (@) virt extra_virt (*no dups allowed*))
+	     all_rules (List.rev extra_rules),
+	   List.fold_left (@) virt extra_virt (*no dups allowed*),
+	   (all_metas : 'a list))
       |	_ -> failwith "unexpected code before the first rule\n")
   | (false,[(PC.TArobArob,_)]) | (false,[(PC.TArob,_)]) ->
-      ([],([] : Ast0.parsed_rule list),[] (*virtual rules*))
+      ([],([] : Ast0.parsed_rule list),[] (*virtual rules*), [] (*all metas*))
   | _ -> failwith "unexpected code before the first rule\n" in
   res) in
   parse_loop file
@@ -1716,7 +1772,7 @@ let parse file =
 (* parse to ast0 and then convert to ast *)
 let process file isofile verbose =
   let extra_path = Filename.dirname file in
-  let (iso_files, rules, virt) = parse file in
+  let (iso_files, rules, virt, _metas) = parse file in
   eval_virt virt;
   let std_isos =
     match isofile with
@@ -1727,9 +1783,12 @@ let process file isofile verbose =
   let parsed =
     List.map
       (function
-          Ast0.ScriptRule (a,b,c,d) -> [([],Ast.ScriptRule (a,b,c,d))]
-	| Ast0.InitialScriptRule (a,b,c) -> [([],Ast.InitialScriptRule (a,b,c))]
-	| Ast0.FinalScriptRule (a,b,c) -> [([],Ast.FinalScriptRule (a,b,c))]
+          Ast0.ScriptRule (a,b,c,d,e) ->
+	    [([],Ast.ScriptRule (a,b,c,d,e))]
+	| Ast0.InitialScriptRule(a,b,c,d) ->
+	    [([],Ast.InitialScriptRule (a,b,c,d))]
+	| Ast0.FinalScriptRule (a,b,c,d) ->
+	    [([],Ast.FinalScriptRule (a,b,c,d))]
 	| Ast0.CocciRule
 	    ((minus, metavarsm,
 	      (iso, dropiso, dependencies, rule_name, exists)),
@@ -1828,11 +1887,8 @@ let process file isofile verbose =
   if !Flag_parsing_cocci.show_SP
   then List.iter Pretty_print_cocci.unparse code;
 
-  let grep_tokens =
-    Common.profile_code "get_constants" (* for grep *)
-      (fun () -> Get_constants.get_constants code) in
-  let glimpse_tokens2 =
+  let (grep_tokens,glimpse_tokens) =
     Common.profile_code "get_glimpse_constants" (* for glimpse *)
       (fun () -> Get_constants2.get_constants code neg_pos) in
 
-  (metavars,code,fvs,neg_pos,ua,pos,grep_tokens,glimpse_tokens2)
+  (metavars,code,fvs,neg_pos,ua,pos, grep_tokens,glimpse_tokens)
