@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -32,5 +34,5 @@ val process :
 	     (*fresh used after list seeds*)
 	     Ast_cocci.meta_name list list list) *
 	  Ast_cocci.meta_name list list list (*positions list*) *
-	  string list list (* non metavars in - code, for grep *) *
+	  string list option (* non metavars in - code, for grep *) *
 	  string list option (* non metavars in - code, for glimpse/google *)
