@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -168,8 +170,11 @@ let combiner bind option_default
       | Ast.MetaExprList(name,_,_,_) -> meta_mcode name
       | Ast.EComma(cm) -> string_mcode cm
       | Ast.DisjExpr(exp_list) -> multibind (List.map expression exp_list)
-      | Ast.NestExpr(expr_dots,whencode,multi) ->
-	  bind (expression_dots expr_dots) (get_option expression whencode)
+      | Ast.NestExpr(starter,expr_dots,ender,whencode,multi) ->
+	  bind (string_mcode starter)
+	    (bind (expression_dots expr_dots)
+	       (bind (string_mcode ender)
+		  (get_option expression whencode)))
       | Ast.Edots(dots,whencode) | Ast.Ecircles(dots,whencode)
       | Ast.Estars(dots,whencode) ->
 	  bind (string_mcode dots) (get_option expression whencode)
@@ -271,7 +276,7 @@ let combiner bind option_default
       match Ast.unwrap i with
 	Ast.MetaInit(name,_,_) -> meta_mcode name
       |	Ast.InitExpr(exp) -> expression exp
-      | Ast.InitList(lb,initlist,rb,whencode) ->
+      | Ast.InitList(allminus,lb,initlist,rb,whencode) ->
 	  multibind
 	    [string_mcode lb;
 	      multibind (List.map initialiser initlist);
@@ -434,9 +439,12 @@ let combiner bind option_default
       | Ast.Atomic(re) -> rule_elem re
       | Ast.Disj(stmt_dots_list) ->
 	  multibind (List.map statement_dots stmt_dots_list)
-      | Ast.Nest(stmt_dots,whn,_,_,_) ->
-	  bind (statement_dots stmt_dots)
-	    (multibind (List.map (whencode statement_dots statement) whn))
+      | Ast.Nest(starter,stmt_dots,ender,whn,_,_,_) ->
+	  bind (string_mcode starter)
+	    (bind (statement_dots stmt_dots)
+	       (bind (string_mcode ender)
+		  (multibind
+		     (List.map (whencode statement_dots statement) whn))))
       | Ast.FunDecl(header,lbrace,body,rbrace) ->
 	  multibind [rule_elem header; rule_elem lbrace;
 		      statement_dots body; rule_elem rbrace]
@@ -675,8 +683,9 @@ let rebuilder
 	    Ast.MetaExprList(meta_mcode name,lenname_inh,keep,inherited)
 	| Ast.EComma(cm) -> Ast.EComma(string_mcode cm)
 	| Ast.DisjExpr(exp_list) -> Ast.DisjExpr(List.map expression exp_list)
-	| Ast.NestExpr(expr_dots,whencode,multi) ->
-	    Ast.NestExpr(expression_dots expr_dots,
+	| Ast.NestExpr(starter,expr_dots,ender,whencode,multi) ->
+	    Ast.NestExpr(string_mcode starter,expression_dots expr_dots,
+			 string_mcode ender,
 			 get_option expression whencode,multi)
 	| Ast.Edots(dots,whencode) ->
 	    Ast.Edots(string_mcode dots,get_option expression whencode)
@@ -765,8 +774,9 @@ let rebuilder
 	  Ast.MetaInit(name,keep,inherited) ->
 	    Ast.MetaInit(meta_mcode name,keep,inherited)
 	| Ast.InitExpr(exp) -> Ast.InitExpr(expression exp)
-	| Ast.InitList(lb,initlist,rb,whencode) ->
-	    Ast.InitList(string_mcode lb, List.map initialiser initlist,
+	| Ast.InitList(allminus,lb,initlist,rb,whencode) ->
+	    Ast.InitList(allminus,
+			 string_mcode lb, List.map initialiser initlist,
 			 string_mcode rb, List.map initialiser whencode)
 	| Ast.InitGccName(name,eq,ini) ->
 	    Ast.InitGccName(ident name, string_mcode eq, initialiser ini)
@@ -943,8 +953,9 @@ let rebuilder
 	| Ast.Atomic(re) -> Ast.Atomic(rule_elem re)
 	| Ast.Disj(stmt_dots_list) ->
 	    Ast.Disj (List.map statement_dots stmt_dots_list)
-	| Ast.Nest(stmt_dots,whn,multi,bef,aft) ->
-	    Ast.Nest(statement_dots stmt_dots,
+	| Ast.Nest(starter,stmt_dots,ender,whn,multi,bef,aft) ->
+	    Ast.Nest(string_mcode starter,statement_dots stmt_dots,
+		     string_mcode ender,
 		     List.map (whencode statement_dots statement) whn,
 		     multi,bef,aft)
 	| Ast.FunDecl(header,lbrace,body,rbrace) ->
