@@ -1,5 +1,7 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2010, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -21,6 +23,7 @@
 
 
 open Common
+open Sexplib
 
 (*****************************************************************************)
 (* Test framework *)
@@ -457,11 +460,23 @@ let test_parse_cocci file =
   let (_,xs,_,_,_,_,grep_tokens,query) =
     Parse_cocci.process file (Some !Config.std_iso) false in
   xs +> List.iter Pretty_print_cocci.unparse;
+  Format.print_newline();
+  (* compile ocaml script code *)
+  (match Prepare_ocamlcocci.prepare file xs with
+       None -> ()
+     | Some ocaml_script_file ->
+	 (* compile file *)
+	 Prepare_ocamlcocci.load_file ocaml_script_file;
+	 (* remove file *)
+	 Prepare_ocamlcocci.clean_file ocaml_script_file;
+	 (* Print the list of registered functions *)
+	 Prepare_ocamlcocci.test ());
   Printf.printf "grep tokens\n";
-  List.iter (function x -> Printf.printf "%s\n" (String.concat " " x))
-    grep_tokens;
+  (match grep_tokens with
+    None -> pr "No query"
+  | Some x -> pr (String.concat " || " x));
   match !Flag.scanner with
-    Flag.NoScanner -> ()
+    Flag.NoScanner | Flag.Grep -> ()
   | Flag.Glimpse | Flag.Google _ ->
       (match query with
 	None -> pr "No query"
